''**************************************
''* DEMO.spin                   2/2016 *
''* Author: Werner L. Schneider        *
''**************************************
''
''
CON
    _clkmode      = xtal1 + pll16x
    _xinfreq      = 5_000_000

    video    = 16                      ' P16-P23 VGA

  
VAR
 
    byte vgabuff[4800]                 ' VGA Text-Buffer 80x30 2400 x 2 Bytes 1. Byte = Character, 2. Byte = Color (bgc * 16 +fgc)
    byte cursor[6]                     ' Cursor info array 
    long sync                          ' sync used by VGA routine
 
    long pos                           ' Global Screen-Pointer
        

OBJ
    
    vga:    "VGACOLOR_640x480"         ' VGA Driver                               need 2 Cog's


PUB main | i     
    Init


    str(0, 0, $0F, $00, @strHello)             ' Print "Hello World" white on black
    str(0, 2, $0F, $00, @strHello)             ' Print "Hello World" white on black
    repeat i from 0 to 4
        fill(i, 2, i+1, $00)
    repeat i from 6 to 10
        fill(i, 2, i+3, $00)

    str(0, 4, $00, $07, @strHello)             ' Print "Hello World" black on grey
    str(0, 6, $00, $07, @strHello)             ' Print "Hello World" black on grey
    repeat i from 0 to 4
        fill(i, 6, i+1, $07)
    repeat i from 6 to 10
        fill(i, 6, i+3, $07)


    dec(0, 8, $0B, $04, $7FFFFFFF)             ' Print decimal
    dec(0, 10, $0B, $04, $FFFFFFFF)            ' Print decimal

    bin(0, 12, $0B, $02, $7FFFFFFF, 32)        ' Print binary
    bin(0, 14, $0B, $02, $FFFFFFFF, 32)        ' Print binary
    
    hex(0, 16, $0B, $01, $7FFFFFFF, 8)         ' Print hexadecimal
    hex(0, 18, $0B, $01, $FFFFFFFF, 8)         ' Print hexadecimal


PRI Init | i, j
''************************************************
''* Start VGA Cog's, Clear Screen and            *
''* set Cursor's                                 *
''************************************************

    vga.start(video, @vgabuff, @cursor, @sync)     ' Start Video Driver
    waitcnt(clkfreq * 1 + cnt)                     ' wait 1 second for cogs to start

    wordfill(@vgabuff, $0F20, 2400)                ' Clear Screen with $20 Foreground-Color $F (white), Background-Color $0 (black)

    cursor[0] := 11
    cursor[1] := 0
    cursor[2] := %010

    cursor[3] := 11
    cursor[4] := 2
    cursor[5] := %111


PRI fill(x, y, fgc, bgc) | color
''************************************************
''* Fill x, y with Foreground-Color fgc and      *
''*                Background-Color bgc          *
''************************************************

    pos := ((x + y * 80) * 2) + 1
    color := fgc + (bgc * 16)
    vgabuff[pos] := color


PRI print(fgc, bgc, char) 
''************************************************
''* Print Character char at current pos          *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    vgabuff[pos++] := char
    vgabuff[pos++] := bgc * 16 + fgc


PRI str(x, y, fgc, bgc, string_ptr)
''************************************************
''* Print String at x, y                         *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 80) * 2
    repeat strsize(string_ptr)
        print(fgc, bgc, byte[string_ptr++])


PRI dec(x, y, fgc, bgc, value) | i
''************************************************
''* Print decimal value at x, y                  *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 80) * 2

    if value < 0
        -value
         print(fgc, bgc, "-")

    i := 1_000_000_000
    repeat 10
        if value => i
            print(fgc, bgc, value/i + "0")
            value //= i
            result~~
        elseif result or i == 1
            print(fgc, bgc, "0")
        i /= 10     


PRI bin(x, y, fgc, bgc, value, digits)
''************************************************
''* Print binary value at x, y                   *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 80) * 2
    value <<= 32 - digits
    repeat digits
        print(fgc, bgc, (value <-= 1) & 1 + "0")


PRI hex(x, y, fgc, bgc, value, digits)
''************************************************
''* Print hexadecimal value at x, y              *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 80) * 2
    value <<= (8 - digits) << 2
    repeat digits
        print(fgc, bgc, lookupz((value <-= 4) & $f : "0".."9", "A".."F"))


DAT

strHello          byte "Hello World", 0



{{

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}

                       '